# -*- coding: utf-8 -*-
"""
/***************************************************************************
 AddTraversePointsDialog
                                 A QGIS plugin
 Adds new points by azimuth and distance
                             -------------------
        begin                : 2013-10-23
        copyright            : (C) 2013 by CHL
        email                : christina_ludwig@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from PyQt5 import uic
import os
import warnings

import subprocess

from qgis.PyQt.QtCore import Qt, QSettings, QVariant, QDir
from qgis.PyQt.QtWidgets import QMessageBox, QFileDialog, QDialog
from qgis.PyQt.QtGui import QFont

from qgis.core import (Qgis,
                        QgsVectorLayer,
                       QgsProject,
                       QgsApplication,
                       QgsProcessingUtils,
                       QgsProcessingParameterDefinition,
                       QgsProcessingAlgRunnerTask,
                       QgsProcessingOutputHtml,
                       QgsProcessingParameterVectorDestination,
                       QgsProcessingOutputLayerDefinition,
                       QgsProcessingParameterFeatureSink,
                       QgsProcessingParameterRasterDestination,
                       QgsProcessingAlgorithm,
                       QgsProxyProgressTask,
                       QgsTaskManager, QgsPalLayerSettings, QgsTextFormat, QgsVectorLayerSimpleLabeling)

from qgis.gui import (QgsGui,
                      QgsMessageBar,
                      QgsEncodingFileDialog,
                      QgsProcessingAlgorithmDialogBase,
                      QgsEncodingFileDialog,
                      QgsMapToolIdentifyFeature)
from qgis.utils import iface

from processing.gui.ParametersPanel import ParametersPanel
from processing.gui.BatchAlgorithmDialog import BatchAlgorithmDialog


pluginPath = os.path.dirname(__file__)
with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=DeprecationWarning)
    WIDGET, BASE = uic.loadUiType(os.path.join(os.path.dirname(__file__), 'ui_hyperlink.ui'))

from .help_dialog import HelpDialog


class HyperlinkDialog(BASE, WIDGET):


    def __init__(self, alg):
        """Constructor."""
        super(HyperlinkDialog, self).__init__(None)
        self.setWindowFlags(Qt.WindowStaysOnTopHint)
        self.setupUi(self)

        self.alg = alg

        self.feedback_dialog = None
        self.active_layer = None

        self.context = None
        self.feedback = None
        self.algorith = None
        self.layer = None
        self.canvas = iface.mapCanvas()

        self.help_dlg = HelpDialog()
        self.clickTool = QgsMapToolIdentifyFeature(self.canvas)

        self.btnHelp.clicked.connect(self.help_dlg.show)
        self.cbx_layers.currentIndexChanged.connect(self.populate_attribute_box)
        self.chk_labels.stateChanged.connect(self.toggle_labels)
        self.chk_tool.stateChanged.connect(self.toggle_tool)
        self.btnBrowse.clicked.connect(self.relative_link)
        QgsProject().instance().layerRemoved.connect(self.populate_layer_combobox)
        QgsProject().instance().layerWasAdded.connect(self.populate_layer_combobox)

        self.manageGui()

    def manageGui(self):
        self.populate_layer_combobox()
        self.populate_attribute_box()

    def setAlgorithm(self, alg):
        self.algorithm = alg

    def getParametersPanel(self, alg, parent):
        return ParametersPanel(parent, alg, self.in_place)

    def runAsBatch(self):
        self.close()
        dlg = BatchAlgorithmDialog(self.algorithm().create(), parent=iface.mainWindow())
        dlg.show()
        dlg.exec_()

    def setParameters(self, parameters):
        self.mainWidget().setParameters(parameters)

    def populate_layer_combobox(self):
        """
        Add all polygon vector layers to the change box
        :return:
        """
        project = QgsProject().instance()
        layers = []
        for lay in project.mapLayers():
            if isinstance(project.mapLayer(lay), QgsVectorLayer):
                layers.append(project.mapLayer(lay).name())
        self.cbx_layers.clear()
        self.cbx_layers.addItems(layers)

    def populate_attribute_box(self):
        """
        Polulate combo box with attributes of layer
        :return:
        """
        self.selected_layer()
        if isinstance(self.layer, QgsVectorLayer):
            self.cbx_label.clear()
            self.cbx_link.clear()
            for field in self.layer.fields():
                self.cbx_label.addItem(field.name())
                self.cbx_link.addItem(field.name())
        return 0

    def selected_layer(self):
        """
        Get the selected layer
        :return:
        """
        layers = QgsProject().instance().mapLayersByName(self.cbx_layers.currentText())
        if layers:
            self.layer = layers[0]
        else:
            self.layer = None
        return self.layer

    def relative_link(self):
        """
        Get the path to the link directory
        :return:
        """
        self.ldtRelativePath.clear()
        self.directory = self.choose_directory_dialog()[0]
        if self.directory:
            self.ldtRelativePath.setText(self.directory)

    def choose_directory_dialog(self):
        """
        Open file dailog so user can select the template
        :return: (User selected file, encoding)
        """
        default_dir = QSettings().value("/UI/lastShapefileDir")
        encoding = QSettings().value("/UI/encoding")

        file_dialog = QgsEncodingFileDialog(self, "Ordner wählen", default_dir, encoding)
        file_dialog.setFileMode(QFileDialog.Directory)
        #file_dialog.setAcceptMode(QFileDialog.AcceptOpen)
        file_dialog.setOption(QFileDialog.ShowDirsOnly)
        if not file_dialog.exec_() == QDialog.Accepted:
            return None, None

        print(file_dialog.selectedFiles()[0])
        folder_path = QDir(file_dialog.selectedFiles()[0]).absolutePath()
        QSettings().setValue("/UI/lastShapefileDir", folder_path)

        return folder_path, file_dialog.encoding()


    def toggle_labels(self):
        """
        Toggle labels on or off
        :return:
        """
        self.selected_layer()
        if self.layer is not None:
            if self.chk_labels.checkState() == Qt.Checked:

                layer_pal_settings = QgsPalLayerSettings()
                text_format = QgsTextFormat()
                font = QFont("Arial", 12)
                font.setBold(True)
                text_format.setFont(font)
                layer_pal_settings.setFormat(text_format)

                layer_pal_settings.fieldName = self.cbx_label.currentText()
                layer_pal_settings.placement = 2
                layer_pal_settings.enabled = True

                layer_settings = QgsVectorLayerSimpleLabeling(layer_pal_settings)
                self.layer.setLabelsEnabled(True)
                self.layer.setLabeling(layer_settings)
                self.layer.triggerRepaint()
                self.canvas.refresh()
            else:
                self.layer.setLabelsEnabled(False)
                self.canvas.refresh()

    def toggle_tool(self):
        """
        Toggle click tool on or off
        :return:
        """
        if self.chk_tool.checkState() == Qt.Checked:
            self.clickTool.setLayer(self.layer)
            # connect our select function to the canvasClicked signal
            self.clickTool.featureIdentified.connect(self.open_link)
            self.canvas.setMapTool(self.clickTool)
        else:
            # disconnect our select function to the canvasClicked signal
            self.clickTool.featureIdentified.disconnect(self.open_link)
            self.canvas.unsetMapTool(self.clickTool)

    def open_link(self, point):
        """
        Open linked file
        :param point:
        :return:
        """
        link_path = point.attribute(self.cbx_link.currentText())
        if isinstance(link_path, QVariant):
            if link_path.isNull():
                return 0

        if self.ckbRelativePath.checkState() == Qt.Checked:
            self.directory = self.ldtRelativePath.text()
            link_path = os.path.join(self.directory, link_path)

        print(link_path)
        if not isinstance(link_path, str):
            return 0

        if not os.path.exists(link_path):
            return 0

        #link_path.replace("/", "\\" )

        if os.name == "posix":
            subprocess.Popen(["open", link_path])
        else:
            result = os.startfile(link_path)
            if result == 0:
                QMessageBox.warning(iface.mainWindow(), "Info", u"Kein gültiger Hyperlink gefunden.\n\nStellen Sie sicher, dass der richtige Feldname ausgewält ist und keine Sonder- oder Leerzeichen im Dateinamen enthalten sind.")

